"""Internal implementation details.

.. warning::

    This package is intended only for internal use within Accretion.
    This documentation is provided for information purposes only.
    No guarantee is provided on the modules and APIs described in here remaining consistent.
    Directly reference at your own risk.
"""
__all__ = ("__version__",)

__version__ = "0.0.1b0"
